//
//  FMFireSequeAnimation.m
//  Storyboards
//
//  Created by Mark Brindle on 06/01/2012.
//  Copyright (c) 2012 ARKEMM Software Limited. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import <CoreImage/CoreImage.h>

#import "FMFireSequeAnimation.h"

void FMSetGasValueForFireEmmitterLayer(double gas, CAEmitterLayer *fireEmitter) {
  [fireEmitter setValue:[NSNumber numberWithInt:(int)(550 * gas)] forKeyPath:@"emitterCells.fire.birthRate"];
  [fireEmitter setValue:[NSNumber numberWithFloat:gas] forKeyPath:@"emitterCell.fire.lifetime"];
  [fireEmitter setValue:[NSNumber numberWithFloat:(0.1 + gas * 0.1)] forKeyPath:@"emitterCells.fire.lifetimeRange"];
  CGColorRef fireColour = [[UIColor colorWithRed:0.8 green:0.4 blue:0.2 alpha:0.15 + gas * gas * 0.05] CGColor];
  [fireEmitter setValue:objc_unretainedObject(fireColour) forKeyPath:@"emitterCells.fire.color"];
  fireEmitter.emitterSize = CGSizeMake(300 * gas + 20, 0);
}

void FMPerformVisualFireTransitionAttachedToLayerWithDuration(CALayer *layer, NSTimeInterval secondsToBurn) {
  CAEmitterLayer *fireEmitter = [CAEmitterLayer layer];
  
  // Create the fire emitter layer
  fireEmitter.emitterPosition = CGPointMake(160, 260);
  fireEmitter.emitterMode = @"outline";
  fireEmitter.emitterShape = @"line";
  fireEmitter.renderMode = @"additive";
  
  // Create the fire emitter cell
  CAEmitterCell *fire = [CAEmitterCell emitterCell];
  fire.emissionLongitude = 0;
  fire.velocity = 1200;
  fire.velocityRange = 300;
  fire.emissionRange = 0.4;
  fire.yAcceleration = -550;
  fire.xAcceleration = 350;
  fire.scale = 5;
  fire.alphaSpeed = 0.2;
  fire.scaleSpeed = 0.5;
  fire.contents = objc_unretainedObject([[UIImage imageNamed:@"fire.png"] CGImage]);
  
  // Name the cell so it can be animated later using keypaths
  [fire setName:@"fire"];
  
  // Add the fire emitter cell to the fire emitter layer
  fireEmitter.emitterCells= [NSArray arrayWithObject:fire];
  [layer addSublayer:fireEmitter];
  CATransform3DMakeTranslation(0, -120, 0);
  
  // Light the fire
  FMSetGasValueForFireEmmitterLayer(1.0, fireEmitter);
  dispatch_time_t gasTime = dispatch_time(DISPATCH_TIME_NOW, secondsToBurn * NSEC_PER_SEC);
  dispatch_after(gasTime, dispatch_get_main_queue(), ^(void){
    FMSetGasValueForFireEmmitterLayer(0.0, fireEmitter);
  });
}
